macro_rules! declare_schema {
($name:ident = $id:literal) => { ... };
}Expand description
Declare a schema with a given name and id.
macro_rules! declare_schema {
($name:ident = $id:literal) => { ... };
}Declare a schema with a given name and id.